import LabelText from 'atoms/LabelText';
import ENTITY_FORM_LIST from 'constant/templates/form/entity_form_template';
import useCreatingFormData from 'hooks/creating_form_data';
import useDrupalTranslation from 'hooks/drupal_translation';
import EntityFormTemplate from 'model/EntityFormTemplate';
import AlertMessages from 'molecules/AlertMessages';
import Breadcrumbs from 'organisms/Breadcrumbs';
import EntityRecordForm from 'organisms/EntityRecordForm';
import PageTitle from 'organisms/PageTitle';
import SideBar from 'organisms/SideBar';
import { useContext, useEffect, useState } from 'react';
import { Button, Col, Container, Form, Row } from 'react-bootstrap';
import { useHistory, useParams } from 'react-router-dom';
import { AppContext } from 'service/state';

const TemplateCreatePageImpl = ({cloudContext, entityFormTemplate}: {
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate
}) => {
  const { formData, setFormData } = useCreatingFormData(cloudContext, entityFormTemplate);
  const { t } = useDrupalTranslation();
  const { addMessage, addErrorMessage } = useContext(AppContext);
  const [isRefreshing, setRefreshing] = useState(false);
  const history = useHistory();

  useEffect(() => {
    console.group('FormData');
    console.log(formData);
    console.groupEnd();
  }, [formData]);

  /**
   * Save entity's data.
   */
  const save = async () => {
    setRefreshing(true);
    const url = `/cloud_dashboard/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/cloud_launch_template/create`;

    const response = await fetch(url, {
      method: 'POST',
    });
    setRefreshing(false);
    if (!response.ok) {
      addErrorMessage(t('Unable to create.'));
      console.group('Create Request');
      console.log('Result: Failed');
      console.log('Response:');
      console.error(await response.json());
      console.groupEnd();
    } else {
      addMessage(t('Created successfully.'));
      console.group('Create Request');
      console.log('Result: Success');
      console.log('Response:');
      console.log(await response.json());
      console.groupEnd();
      history.push(`/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/server_template`);
    }
  }

  return <>
    <AlertMessages />
    <PageTitle />
    <Breadcrumbs />
    <SideBar />
    <Container fluid className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            {
              entityFormTemplate.entityRecords.map((entityRecord, index) => {
                return <EntityRecordForm key={index}
                  cloudServiceProvider={entityFormTemplate.cloudServiceProvider}
                  cloudContext={cloudContext}
                  entityName="server_template"
                  action={entityFormTemplate.actionType}
                  formData={formData}
                  setFormData={setFormData}
                  entityRecord={entityRecord} />;
              })
            }
            <Button variant={isRefreshing ? 'secondary' : 'primary'}
              disabled={isRefreshing}
              className="ripple-effect" onClick={save}>
              <LabelText text="Save" />
            </Button>
          </Form>
        </Col>
      </Row>
    </Container>
  </>;
}

const TemplateCreatePage = () => {

  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  const entityFormTemplateList = ENTITY_FORM_LIST.filter((template) => {
    return template.cloudServiceProvider === params.cloudServiceProvider
      && template.entityName === 'server_template'
      && template.actionType === 'create'
  });
  if (entityFormTemplateList.length === 0) {
    return <>
      <PageTitle />
      <Breadcrumbs />
      <SideBar />
      <Container fluid className="px-0">
        <Row>
          <Col>
            <span>TemplateCreatePage</span><br />
            <span>cloudServiceProvider={params.cloudServiceProvider}</span><br />
            <span>cloudContext={params.cloudContext}</span><br />
            <span>entityName=server_template</span><br />
            <span>action=create</span>
          </Col>
        </Row>
      </Container>
    </>;
  }
  const entityFormTemplate = entityFormTemplateList[0];

  return <TemplateCreatePageImpl cloudContext={params.cloudContext} entityFormTemplate={entityFormTemplate} />;
}

export default TemplateCreatePage;
